@@ -12,9 +12,7 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * License along with this library; if not, see <http://www.gnu.org/licenses/>.
  *
  * Author: Ryan Lortie <desrt@desrt.ca>
  */
@@ -22,7 +20,7 @@
 #ifndef __dconf_h__
 #define __dconf_h__
 
-#include <common/dconf-error.h>
+#include <common/dconf-enums.h>
 #include <common/dconf-paths.h>
 #include <common/dconf-changeset.h>
 #include <client/dconf-client.h>
